package Zucchini::Contributors;
# vim: ts=8 sts=4 et sw=4 sr sta
use strict;
use warnings;

use Zucchini::Version; our $VERSION = $Zucchini::VERSION;

# this is a dummy package to provide POD

1;

__END__

=pod

=head1 NAME

Zucchini::Contributors - people that helped out along the way

=head1 AUTHOR

Chisel Wright C<< <chiselwright@users.berlios.de> >>

=head1 INTERFACE

=head2 Jason Tang

Refactoring of ttsite script

=head1 PROJECTS

L<Template>

=cut
